==> xsde-1.1.0-sparc-solaris.tar.gz: xsde-1.1.0-sparc-solaris/libxsde/xsde/cxx/parser/validating/negative-integer.cxx <==
// file      : xsde/cxx/parser/validating/negative-integer.cxx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : Copyright (c) 2005-2007 Code Synthesis Tools CC
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

#include <stdlib.h> // strtoul
#include <limits.h> // LONG_MIN

#include <xsde/cxx/errno.hxx>

#include <xsde/cxx/parser/validating/negative-integer.hxx>

namespace xsde
{
  namespace cxx
  {
    namespace parser
    {
      namespace validating
      {
        void negative_integer_pimpl::
        _pre ()
        {
          size_ = 0;
          sign_ = none;
          state_ = leading_ws;
        }

        void negative_integer_pimpl::
        _characters (const ro_string& s)
        {
          if (!parse (s, str_, 20))
            schema_error (schema_error::invalid_negative_integer_value);
        }

        void negative_integer_pimpl::
        _post ()
        {
          ro_string tmp (str_, size_);
          size_t size = trim_right (tmp);

          if (sign_ == minus && size != 0 && tmp[0] != '-' && tmp[0] != '+')
          {
            str_[size] = '\0';

            char* p;
            set_errno (0);
            unsigned long ul = strtoul (str_, &p, 10);

            if (*p != '\0' ||
                get_errno () != 0 ||
                ul == 0 ||
                ul > static_cast<unsigned long> (LONG_MIN))
              schema_error (schema_error::invalid_negative_integer_value);

            value_ = ul == static_cast<unsigned long> (LONG_MIN)
              ? LONG_MIN : -static_cast<long> (ul);
          }
          else
            schema_error (schema_error::invalid_negative_integer_value);
        }

        long negative_integer_pimpl::
        post_negative_integer ()
        {
          return value_;
        }
      }
    }
  }
}

==> xsde-3.0.0-sparc-solaris.tar.gz: xsde-3.0.0-sparc-solaris/libxsde/xsde/cxx/parser/validating/negative-integer.cxx <==
// file      : xsde/cxx/parser/validating/negative-integer.cxx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : Copyright (c) 2005-2009 Code Synthesis Tools CC
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

#include <stdlib.h> // strtoul
#include <limits.h> // LONG_MIN

#include <xsde/cxx/errno.hxx>

#include <xsde/cxx/parser/validating/negative-integer.hxx>

namespace xsde
{
  namespace cxx
  {
    namespace parser
    {
      namespace validating
      {
        void negative_integer_pimpl::
        _pre ()
        {
          size_ = 0;
          sign_ = none;
          state_ = leading_ws;
        }

        void negative_integer_pimpl::
        _characters (const ro_string& s)
        {
          if (!parse (s, str_, 20))
            _schema_error (schema_error::invalid_negative_integer_value);
        }

        void negative_integer_pimpl::
        _post ()
        {
          ro_string tmp (str_, size_);
          size_t size = trim_right (tmp);

          if (sign_ == minus && size != 0 && tmp[0] != '-' && tmp[0] != '+')
          {
            str_[size] = '\0';

            char* p;
            set_errno (0);
            unsigned long ul = strtoul (str_, &p, 10);

            if (*p != '\0' ||
                get_errno () != 0 ||
                ul == 0 ||
                ul > static_cast<unsigned long> (LONG_MIN))
              _schema_error (schema_error::invalid_negative_integer_value);

            value_ = ul == static_cast<unsigned long> (LONG_MIN)
              ? LONG_MIN : -static_cast<long> (ul);
          }
          else
            _schema_error (schema_error::invalid_negative_integer_value);
        }

        long negative_integer_pimpl::
        post_negative_integer ()
        {
          return value_;
        }
      }
    }
  }
}
